﻿using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.IO;
using Microsoft.SharePoint.Utilities;

namespace AsyncWebPart {

    // Based on the original code from: http://blogs.msdn.com/b/pavankumar/archive/2009/02/16/code-sample-for-a-asynchronous-webpart.aspx
    // Adapted for SP2013
    // Implement RunProcess with the long running code
    // Implement RenderAsync to render the webpart content
    // Use property WaitImageUrl to change the default wait image
    // This code does not work for sandboxed solutions

    [ToolboxItemAttribute(false)]
    public abstract class AsyncWebPartBase : WebPart, ICallbackEventHandler {

        private string _id;
        private string _asyncContent;
        private string _waitImageUrl = "/_layouts/15/IMAGES/gears_anv4.gif"; //Default wait image...

        public string WaitImageUrl { 
            get {
                return _waitImageUrl;
            }
            set {
                _waitImageUrl = value;
            }
        }

        protected override void OnLoad(EventArgs e) {
            base.OnLoad(e);
            _id = this.ID + "_itemsdata";
            string js = Page.ClientScript.GetCallbackEventReference(this, "'getcontent'", "fill" + _id, _id, true);
            string contentloader = "function fill" + _id + "(arg, ctx) { \n\r var content = document.getElementById('" + _id + "');\n\r content.innerHTML = arg;\n\r }";
            if (SPPageContentManager.IsClientScriptBlockRegistered(Page, this.GetType(), "contentloader" + _id) == false)
                SPPageContentManager.RegisterClientScriptBlock(Page, this.GetType(), "contentloader" + _id, contentloader);
            SPPageContentManager.RegisterStartupScript(Page, this.GetType(), "myloader" + _id, "_spBodyOnLoadFunctions.push(function() {" + js + "});");
        }

        protected sealed override void Render(HtmlTextWriter writer) {
            if (!string.IsNullOrEmpty(this.CssClass))
                writer.AddAttribute(HtmlTextWriterAttribute.Class, this.CssClass);
            writer.AddAttribute(HtmlTextWriterAttribute.Id, _id);
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            writer.AddAttribute(HtmlTextWriterAttribute.Alt, "Loading...");
            writer.AddAttribute(HtmlTextWriterAttribute.Src, _waitImageUrl);
            writer.RenderBeginTag(HtmlTextWriterTag.Img);
            writer.RenderEndTag();
            writer.RenderEndTag();
        }

        protected abstract void RunProcesses();

        protected abstract void RenderAsync(HtmlTextWriter writer);

        public void RaiseCallbackEvent(string eventArgument) {
            switch (eventArgument) {
                case "getcontent":
                    using (StringWriter stringWriter = new StringWriter())
                    using (HtmlTextWriter writer = new HtmlTextWriter(stringWriter)) {
                        EnsureChildControls();
                        RunProcesses();
                        RenderAsync(writer);
                        _asyncContent = stringWriter.ToString();
                    }
                    break;
            }
        }

        public string GetCallbackResult() {
            return _asyncContent;
        }

    }

}
